package com.sdk.sdkdemo;

import androidx.appcompat.app.AppCompatActivity;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.hardware.display.DisplayManager;
import android.os.Bundle;
import android.os.SystemClock;
import android.text.TextPaint;
import android.util.Log;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Switch;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;
import android.widget.Toast;

import com.sdklib.Observer;
import com.sdklib.command.Command;
import com.sdklib.command.EscCommand;
import com.sdklib.connection.CashDrawerConnection;
import com.sdklib.io.UsbPort;

import java.io.IOException;
import java.lang.reflect.Field;
import java.text.DecimalFormat;
import java.util.Random;

import static android.graphics.Bitmap.createBitmap;
import static android.hardware.usb.UsbManager.ACTION_USB_DEVICE_ATTACHED;
import static android.hardware.usb.UsbManager.ACTION_USB_DEVICE_DETACHED;

/**
 * Created by Administrator
 *
 * @author 猿史森林
 * Date: 2019/9/21
 * Class description:
 */
public class MainActivity extends AppCompatActivity {

    private static final String TAG = MainActivity.class.getSimpleName();
    /**
     * ESC Query printer real-time status.Out of paper status
     */
    private static final int ESC_STATE_PAPER_ERR = 0x20;

    /**
     * ESC Query printer real-time status.Cover open status
     */
    private static final int ESC_STATE_COVER_OPEN = 0x04;

    /**
     * ESC Query printer real-time status.Printer error
     */
    private static final int ESC_STATE_ERR_OCCURS = 0x40;
    private EscCommand escCommand;
    private CashDrawerConnection drawerConnection;
    private DisplayManager displayManager;
    private Customer customer;
    private boolean startOrEndSecondaryScreen;
    private Button btnCustomer;
    private ThreadFactoryBuilder threadFactoryBuilder = new ThreadFactoryBuilder("main_activity");
    private EditText etBlackMarkWidth;
    private EditText etBlackMarkPagerLength;
    private EditText etBlackMarkPrintPosition;
    private EditText etBlackMarkCutterPosition;
    private MyBroadcastReceiver receiver;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        displayManager = (DisplayManager) getSystemService(DISPLAY_SERVICE);
        //bind cash service
        Intent intent = new Intent("com.android.settings.EasyService");
        intent.setPackage("com.android.settings");
        drawerConnection = new CashDrawerConnection();
        bindService(intent, drawerConnection, Context.BIND_AUTO_CREATE);

        receiver = new MyBroadcastReceiver();
        IntentFilter filter = new IntentFilter();
        filter.addAction(ACTION_USB_DEVICE_DETACHED);
        this.registerReceiver(receiver, filter);

        btnCustomer = findViewById(R.id.customer);
        btnCustomer.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!startOrEndSecondaryScreen) {
                    showSecondaryScreen();
                    startOrEndSecondaryScreen = true;
                    btnCustomer.setText(R.string.str_close_customer);
                } else {
                    startOrEndSecondaryScreen = false;
                    customer.dismiss();
                    btnCustomer.setText(R.string.str_open_customer);
                }
            }
        });

        Switch swTurnOnOrOffCut = findViewById(R.id.sw_cut);
        swTurnOnOrOffCut.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                cutterTurnOnOrOff(isChecked);
            }
        });

        Switch swTurnOnOrOffBlackMark = findViewById(R.id.sw_turn_on_or_off_black_mark);
        swTurnOnOrOffBlackMark.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                blackMarkModeTurnOnOrOff(isChecked);
            }
        });

        etBlackMarkPagerLength = findViewById(R.id.et_setting_black_mark_pager_length);
        etBlackMarkWidth = findViewById(R.id.et_setting_black_mark_width);
        etBlackMarkPrintPosition = findViewById(R.id.et_setting_start_print_position);
        etBlackMarkCutterPosition = findViewById(R.id.et_setting_cutter_position);
    }

    public void blackMarkModeTurnOnOrOff(final boolean isCheck) {
        ThreadPool.getInstantiation().addTask(threadFactoryBuilder.newThread(new Runnable() {
            @Override
            public void run() {
                connect(new ConnectState() {
                    @Override
                    public void success() {
                        escCommand.addBlackMark(isCheck);
                    }
                });
            }
        }));
    }

    public void cutterTurnOnOrOff(final boolean status) {
        ThreadPool.getInstantiation().addTask(threadFactoryBuilder.newThread(new Runnable() {
            @Override
            public void run() {
                connect(new ConnectState() {
                    @Override
                    public void success() {
                        try {
                            //读取切刀状态
                            escCommand.addUserCommand(new byte[]{0x1f, 0x1b, 0x1f, (byte) 0xa1, 0x10, 0x11, 0x12, 0x13, 0x14, 0x16, 0x11});
                            byte[] buffer = new byte[4];
                            int len = escCommand.read(buffer, 2);
                            if (len > 0) {
                                byte result = buffer[0];
                                int cutter = result & 0x1;
                                int density = result >> 2 & 0x1;
                                int characterMode = result >> 3 & 0x1;
                                int aLineNumber = result >> 4 & 0x1;
                                byte n = (byte) (aLineNumber & 0x1 << 4 | characterMode & 0x1 << 3 | density & 0x1 << 2 | 1 & 0x1 << 1 | (status ? 1 : 0) & 0x1);
                                escCommand.addUserCommand(new byte[]{0x1f, 0x1b, 0x1f, (byte) 0xa1, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, n});
                            }
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                });
            }
        }));
    }


    public void setBlackMarkPagerLength(View view) {
        connect(new ConnectState() {
            @Override
            public void success() {
                String blackMarkPagerLength = etBlackMarkPagerLength.getText().toString().trim();
                if (blackMarkPagerLength.length() > 0) {
                    escCommand.addBlackMarkPagerLength(Integer.parseInt(blackMarkPagerLength));
                }
            }
        });
    }

    public void setBlackMarkWidth(View view) {
        connect(new ConnectState() {
            @Override
            public void success() {
                String blackMarkWidth = etBlackMarkWidth.getText().toString().trim();
                if (blackMarkWidth.length() > 0) {
                    escCommand.addBlackMarkWidth(Integer.parseInt(blackMarkWidth));
                }
            }
        });
    }

    public void setBlackMarkStartPrintPosition(View view) {
        connect(new ConnectState() {
            @Override
            public void success() {
                String blackMarkPrintPosition = etBlackMarkPrintPosition.getText().toString().trim();
                if (blackMarkPrintPosition.length() > 0) {
                    escCommand.addBlackMarkStartPrintPosition(Integer.parseInt(blackMarkPrintPosition));
                }
            }
        });
    }

    public void setBlackMarkCutterPosition(View view) {
        connect(new ConnectState() {
            @Override
            public void success() {
                String blackMarkCutterPosition = etBlackMarkCutterPosition.getText().toString().trim();
                if (blackMarkCutterPosition.length() > 0) {
                    escCommand.addBlackMarkCutterPosition(Integer.parseInt(blackMarkCutterPosition));
                }
            }
        });
    }

    /**
     * Print custom content
     *
     * @param view
     */
    public void printTest(View view) {
        ThreadPool.getInstantiation().addTask(threadFactoryBuilder.newThread(new Runnable() {
            @Override
            public void run() {
                connect(new ConnectState() {
                    @Override
                    public void success() {
                        printTest();
                    }
                });
            }
        }));
    }

    /**
     * Connect printer
     */
    private void connect(final ConnectState connecter) {
        escCommand = EscCommand.getInstance();
        if (escCommand.isConnected()) {
            connecter.success();
        } else {
            try {
                escCommand.connect(MainActivity.this, new Observer() {
                    @Override
                    public void success() {
                        Log.i(TAG, "success");
                        connecter.success();
                    }

                    @Override
                    public void fail() {
                        Log.e(TAG, "Connection failed");
                    }

                    @Override
                    public void error(Throwable throwable) {
                        Log.e(TAG, "Connection error - " + throwable.toString());
                    }

                    @Override
                    public void other(int code) {
                        Log.e(TAG, "Connection other error - " + code);
                    }
                });
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Print custom content
     */
    private void printTest() {
        if (printerIsNormal()) {
            escCommand.addPrintAndFeedLines((byte) 3);
            // Set print center
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);
            // Set to double height and width
            escCommand.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.ON, EscCommand.ENABLE.ON, EscCommand.ENABLE.OFF);
            // Print text
            escCommand.addText("Sample\n");
            escCommand.addPrintAndLineFeed();

            // Cancel double height and double width
            escCommand.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF);
            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);
            // Print text
            escCommand.addText("Print text\n");
            // Print text
            escCommand.addText("Welcome to use printer!\n");

            /* Printing traditional Chinese requires the printer to support traditional font library */
            String message = "打印繁體中文\n";
            // Print text
            escCommand.addText(message, "GB2312");
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Left");
            //Set horizontal and vertical unit distance
            escCommand.addSetHorAndVerMotionUnits((byte) 7, (byte) 0);
            //Set absolute position
            escCommand.addSetAbsolutePrintPosition((short) 6);
            // Print text
            escCommand.addText("Center");
            //Set absolute position
            escCommand.addSetAbsolutePrintPosition((short) 10);
            // Print text
            escCommand.addText("Right");
            //Paper feed line
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Print bitmap!\n");
            Bitmap b = BitmapFactory.decodeResource(getResources(),
                    R.drawable.printer);
            // Print raster bitmap
            escCommand.addRastBitImage(b, 380, 0);
            b.recycle();

            // Print text
            escCommand.addText("Print code128\n");
            // Set barcode recognizable characters(HRI)
            // Set the position of barcode recognizable characters under the barcode
            escCommand.addSelectPrintingPositionForHRICharacters(EscCommand.HRI_POSITION.BELOW);
            // Set bar code height to 60 points
            escCommand.addSetBarcodeHeight((byte) 60);
            // Set barcode unit width to 1
            escCommand.addSetBarcodeWidth((byte) 1);
            // Print code 128
            escCommand.addCODE128(escCommand.genCodeB("PRINTER"));
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Print QRcode\n");
            // Set error correction level
            escCommand.addSelectErrorCorrectionLevelForQRCode((byte) 0x31);
            // Set QRcode module size
            escCommand.addSelectSizeOfModuleForQRCode((byte) 3);
            // Set QRCode content
            escCommand.addStoreQRCodeData("ABCD1234");
            // Print QRCode
            escCommand.addPrintQRCode();
            escCommand.addPrintAndLineFeed();

            // Set print center alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);
            //Print Thai
            escCommand.addText("Print Thai\r\n");
            escCommand.addText(80, "พิมพ์ภาษาไทย", 24, true, true);
            escCommand.addPrintAndLineFeed();

            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);

            //Print PDF417
            //Print text
            escCommand.addText("Print PDF417\n");
            escCommand.addPdf417PositionCount(10);
            //Set PDF417 module width
            escCommand.addPdf417ModuleWidth(2);
            //Set the PDF417 symbol level height
            escCommand.addPdf417SymbolLevelHeight(8);
            //Set the PDF417 error correction level
            escCommand.addPdf417ErrorCorrectionLevel(49);
            //Set PDF417 options
            escCommand.addPdf417Options(1);
            //Set PDF417 content
            escCommand.addPdf417Content("1234567890");
            //Print PDF417
            escCommand.addPrintPdf417();

            // Set print center alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);
            //Print text
            escCommand.addText("Completed!\r\n");
            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);

            //Feed lines
            escCommand.addPrintAndFeedLines((byte) 8);
            //Cut paper
            escCommand.addCutPaper();
        }
    }

    public void pageModePrintTest(View view) {
        ThreadPool.getInstantiation().addTask(threadFactoryBuilder.newThread(new Runnable() {
            @Override
            public void run() {
                connect(new ConnectState() {
                    @Override
                    public void success() {
                        pageModePrintTest();
                    }
                });
            }
        }));
    }

    private void pageModePrintTest() {
        if (printerIsNormal()) {

            //Switch to page mode
            escCommand.addSelectPageMode();
            //Create print area
            escCommand.addSetPrintArea(0, 0, 256, 300);
            //Print text in standard mode
            escCommand.addText("2) Page Mode Set Area\n" +
                    "( ESC W ) x : 00 y : 00\n" +
                    "\twidth : 256, height : 300, Text T 5LINE, FF\n");
            //Create print area
            escCommand.addSetPrintArea(256, 0, 256, 300);
            escCommand.addSelectErrorCorrectionLevelForQRCode((byte) 0x31);
            escCommand.addSelectSizeOfModuleForQRCode((byte) 3);
            escCommand.addStoreQRCodeData("ABCDEF");
            escCommand.addPrintQRCode();
            escCommand.addPrintAndSwitchStandardMode();
            escCommand.addPrintAndFeedLines((byte) 5);
            escCommand.addCutPaper();
        }
    }

    public static int sp2px(Context context, float spValue) {
        final float fontScale = context.getResources().getDisplayMetrics().scaledDensity;
        return (int) (spValue * fontScale + 0.5f);
    }

    public void printerState(View view) throws IOException {
        connect(new ConnectState() {
            @Override
            public void success() {
                readPrinterState();
            }
        });
    }

    public void blackMarkTest(View view) {
        if (printerIsNormal()) {
            escCommand.addUserCommand(new byte[]{0x1d, 0x0c});
            // Set print center
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.CENTER);
            // Set to double height and width
            escCommand.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.ON, EscCommand.ENABLE.ON, EscCommand.ENABLE.OFF);
            // Print text
            escCommand.addText("Sample\n");
            escCommand.addPrintAndLineFeed();

            // Cancel double height and double width
            escCommand.addSelectPrintModes(EscCommand.FONT.FONTA, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF, EscCommand.ENABLE.OFF);
            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);
            // Print text
            escCommand.addText("Print text\n");
            // Print text
            escCommand.addText("Welcome to use printer!\n");

            /* Printing traditional Chinese requires the printer to support traditional font library */
            String message = "打印繁體中文\n";
            // Print text
            escCommand.addText(message, "GB2312");
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Left");
            //Set horizontal and vertical unit distance
            escCommand.addSetHorAndVerMotionUnits((byte) 7, (byte) 0);
            //Set absolute position
            escCommand.addSetAbsolutePrintPosition((short) 6);
            // Print text
            escCommand.addText("Center");
            //Set absolute position
            escCommand.addSetAbsolutePrintPosition((short) 10);
            // Print text
            escCommand.addText("Right");
            //Paper feed line
            escCommand.addPrintAndLineFeed();

            // Print text
            escCommand.addText("Print code128\n");
            // Set barcode recognizable characters(HRI)
            // Set the position of barcode recognizable characters under the barcode
            escCommand.addSelectPrintingPositionForHRICharacters(EscCommand.HRI_POSITION.BELOW);
            // Set bar code height to 60 points
            escCommand.addSetBarcodeHeight((byte) 60);
            // Set barcode unit width to 1
            escCommand.addSetBarcodeWidth((byte) 1);
            // Print code 128
            escCommand.addCODE128(escCommand.genCodeB("PRINTER"));
            escCommand.addPrintAndLineFeed();

            // Set print left alignment
            escCommand.addSelectJustification(EscCommand.JUSTIFICATION.LEFT);

            //Print text
            escCommand.addText("Completed!\r\n");
            escCommand.addUserCommand(new byte[]{0x0a});
        }
    }

    /**
     * Disconnect
     *
     * @param view
     */
    public void disconn(View view) {
        if (EscCommand.getInstance().isConnected()) {
            escCommand.close();
        }
    }

    /**
     * Receive printer data
     */
    private void readPrinterState() {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                try {
                    Log.i(TAG, "Read data");
                    EscCommand.getInstance().addQueryPrinterStatus();
                    byte[] buffer = new byte[4];
                    int ret = EscCommand.getInstance().read(buffer, 0);
                    String status = App.getContext().getString(R.string.str_printer_conn_normal);
                    if (ret > 0) {
                        if ((buffer[0] & ESC_STATE_PAPER_ERR) > 0) {
                            status += " " + App.getContext().getString(R.string.str_printer_out_of_paper);
                        }
                        if ((buffer[0] & ESC_STATE_COVER_OPEN) > 0) {
                            status += " " + App.getContext().getString(R.string.str_printer_open_cover);
                        }
                        if ((buffer[0] & ESC_STATE_ERR_OCCURS) > 0) {
                            status += " " + App.getContext().getString(R.string.str_printer_error);
                        }
                        System.out.println(App.getContext().getString(R.string.str_state) + status);
                        final String finalStatus = status;
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                Utils.toast(App.getContext(), finalStatus);
                            }
                        });
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        thread.start();
    }

    /**
     * Open cash
     *
     * @param view
     */
    public void openCash(View view) {
        final EscCommand escCommand = EscCommand.getInstance();
        if (escCommand.openCashDrawer(drawerConnection, 1)) {
            Toast.makeText(MainActivity.this, "Cash opened successfully", Toast.LENGTH_SHORT).show();
        } else {
            Toast.makeText(MainActivity.this, "Failed to open the cash", Toast.LENGTH_SHORT).show();
        }
    }

    private void showSecondaryScreen() {
        Display[] displays = displayManager.getDisplays(DisplayManager.DISPLAY_CATEGORY_PRESENTATION);
        if (displays.length > 0) {
            Display display = displays[0];
            if (customer == null) {
                customer = new Customer(this, display);
            }
            customer.show();
            customer.setOnDismissListener(new DialogInterface.OnDismissListener() {
                @Override
                public void onDismiss(DialogInterface dialogInterface) {

                }
            });
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        unbindService(drawerConnection);
        unregisterReceiver(receiver);
    }

    public boolean printerIsNormal() {
        escCommand = EscCommand.getInstance();
        boolean isSuccess = escCommand.addQueryPrinterStatus();
        if (isSuccess) {
            byte[] buffer = new byte[4];
            try {
                int ret = escCommand.read(buffer, 2000);
                if (ret > 0) {
                    if ((buffer[0] & ESC_STATE_PAPER_ERR) == 0 && (buffer[0] & ESC_STATE_COVER_OPEN) == 0 && (buffer[0] & ESC_STATE_ERR_OCCURS) == 0) {
                        return true;
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    private class MyBroadcastReceiver extends BroadcastReceiver {

        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (ACTION_USB_DEVICE_DETACHED.equals(action)) {
                Log.e(TAG, "USB拔出");
                if (EscCommand.getInstance().isConnected()) {
                    EscCommand.getInstance().close();
                }
            }
        }
    }
}
